

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                 *
    *   AutoNumber Proton Plugin 1.2                                  *
    *   Copyright  2004 Sebastian Pipping <webmaster@hartwork.org>   *
    *                                                                 *
    *   --> http://www.hartwork.org                                   *
    *                                                                 *
    *                                                                 *
    *   This source code is released under LGPL.                      *
    *   See LGPL.txt for details.                        2004-09-30   *
    *                                                                 *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */



#include "engine.h"



/******************************************************************************************************
**  engine                                                                                           **
******************************************************************************************************/

bool engine( char * in, char * & out, UINT uExtra )
{
	UINT in_len   = strlen( in );
	UINT out_len  = 0;


	char * szWalker = in;
	bool b13before = false;
	UINT uLines = 0;


	// (I) Count lines (to predict size of output)
	while( *szWalker != '\0' )
	{
		switch( *szWalker )
		{
			case '\x0D':
			{
				b13before = true;

				break;
			}
			case '\x0A':
			{
				if( b13before )
				{
					uLines++;
				}

				b13before = false;

				break;
			}
			default:
			{
				b13before = false;
			}
		}

		szWalker++;
	}

	// Finishing newline?
	bool bNewlineFinish =	( in_len >= 2 ) &&
							( *( szWalker - 2 ) == '\x0D' ) &&
							( *( szWalker - 1 ) == '\x0A' );
	if( !bNewlineFinish )
	{
		uLines++;
	}

	szWalker = in;		// Reset to start



	// Get number of digits
	UINT uBreaks = uLines;
	UINT uDigits = 0;

	while( uBreaks )
	{
		uBreaks /= 10;
		uDigits++;
	}

	if( uDigits == 1 )
	{
		uDigits = 2;
	}
	else if( uDigits > 9 )
	{
		// Error
		return false;
	}

	UINT pre_len = uDigits + 1;


	// Create prefix format
	char szFormat[ 6 ] = "";
	wsprintf( szFormat, "%%0%uu ", uDigits );	// -> "%0xu "


	// Allocate new buffer
	out = new char[ in_len + ( uDigits + 1 ) * uLines + 1 + uExtra ];


	// (II) Create new content
	bool bSearching = true;
	char * szBeg = NULL;
	char * szEnd = NULL;
	char * szBegNext = in;

	for( UINT u = 1; u <= uLines; u++ )
	{
		// (0) Search end of line/string
		b13before   = false;
		bSearching  = true;

		while( bSearching )
		{
			switch( *szWalker )
			{
				case '\x0D':
				{
					b13before = true;

					break;
				}
				case '\x0A':
				{
					if( b13before )
					{
						bSearching = false;		// STOP
						b13before  = false;
					}

					break;
				}
				case '\0':
				{
					bSearching = false;			// STOP
					szWalker--;					// We only walk _one_ past the end

					break;
				}
				default:
				{
					b13before = false;
				}
			}

			szWalker++;
		}


		// (1) Number line
		char * szNumber = new char[ pre_len + 1 ];
		wsprintf( szNumber, szFormat, u );

		memcpy( out + out_len, szNumber, pre_len );
		out_len += pre_len;

		delete szNumber;


		// (2) Copy line
		szBeg = szBegNext;
		szEnd = szWalker;

		UINT line_len = szEnd - szBeg;
		if( line_len > 0 )
		{
			memcpy( out + out_len, szBeg, line_len );
			out_len += line_len;
		}
		szBegNext = szEnd;
	}

	out[ out_len ] = '\0';

	return true;
}
